#!/bin/bash
# Copyright (c) Members of the EGEE Collaboration. 2004.
# See http://www.eu-egee.org/partners/ for details on the copyright holders.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
keydir="/opt/lcg/etc/dpm/https/keystore"
dpmhttpdir="/opt/lcg/var/dpm/https/html";
confdir="/opt/lcg/etc/dpm/https/conf"

usage () {
    echo "dpm-https-conf.sh [--help|-h] [--pool|-p] [--service|-s] [--noservice|-n] [--type|-t <node type>]"
    echo "<node type>   : default 'sor' = headnode & disk server on same node"
    echo "<node type> = 'sor'       = dpm head node & disk server on same node"
    echo "<node type> = 'head-node' = dpm head node"
    echo "<node type> = 'disk-node' = dpm disk node"
    echo "[--pools]     : check the configured data pools on DPM and link them into the html directory";
    echo "[--service]   : add the dpm-httpd service script to run level 345";
    echo "[--noservice] : remove the dpm-httpd service script from run level 345";
 
}

testkeys () {
    case $type in 
	sor|head-node)
	    if [ ! -e "$keydir/key.pem" ]; then
		echo "Error [https-conf]: you miss a private key stored under $keydir/key.pem "
		exit -1;
	    fi
	    ;;

    esac

    case $type in 
	sor|disk-node)
	    if [ ! -e "$keydir/cert.pem" ]; then
		echo "Error [https-conf]: you miss a public key/certificate stored under $keydir/cert.pem "
		exit -1;
	    fi
	    ;;
    esac

    echo "Info [dpm-https-conf]: public & private keys are in $keydir !";
}

keypermissions () {
    case $type in 
	sor|head-node)
	    echo "Info [dpm-https-conf]: setting mode 400 for $keydir/key.pem";
	    chmod 400 $keydir/key.pem;
	    if [ $? != "0" ]; then
		echo "Error [dpm-https-conf]: cannot set mode 400 for $keydir/key.pem";
		exit -1;
	    fi
	    echo "Info [dpm-https-conf]: chown dpmmgr for $keydir/key.pem";
	    chown dpmmgr $keydir/key.pem;
	    if [ $? != "0" ]; then
		echo "Error [dpm-https-conf]: cannot chwon dpmmgr for $keydir/key.pem";
		exit -1;
	    fi
	    ;;
    esac
}

linkpooldirectories () {
    thishost=`hostname -f`;
    for name in `find $dpmhttpdir -type l`; do
	echo "Info [dpm-https-conf]: Cleaning up old link $name ...";
	unlink $name;
    done;
    for name in `/opt/lcg/bin/dpm-qryconf | grep $thishost | awk '{print $2}'`; do 
	echo "Info [dpm-https-conf]: Linking dpm directory $name to http directory $dpmhttpdir ...";
	httplinkdir="$dpmhttpdir/$name";
	httpdir=`dirname $httplinkdir`;
	mkdir -p $httpdir 
	if [ $? != "0" ]; then
	    echo "Error [dpm-https-conf]: cannot create $httpdir";
	    exit -1;
	fi
	echo "Info [dpm-https-conf]: link $httplinkdir->$name";
	test -h $httplinkdir || ln -s $name $httplinkdir;
	if [ $? != "0" ]; then
	    echo "Error [dpm-https-conf]: cannot create link $httplinkdir";
	    exit -1;
	fi
    done;
}

addservice () {
    echo -n "Adding dpm-httpd service ..."
    /sbin/chkconfig --add dpm-httpd
    if [ $? = "0" ]; then
	echo " OK"
    else
	echo " FAILED"
    fi
}

removeservice () {
    echo -n "Removing dpm-httpd service ..."
    /sbin/chkconfig --del dpm-httpd
    if [ $? = "0" ]; then
	echo " OK"
    else
	echo " FAILED"
    fi

}

type="sor";

TEMP=`getopt -o hpsnt: --long help,pool,service,noservice,type: -n 'dpm-https-conf.sh' -- "$@"`
if [ $? != 0 ] ; then usage ; exit 1 ; fi
eval set -- "$TEMP"

if [ "$LOGNAME" != "root" ]; then
   echo "Error: you must run this script as root!";
   exit -1;
fi

while true; do
    case "$1" in
        --help|-h)
            usage
            exit
            ;;
	--type|-t)
	    type="$2"
	    shift 2
	    ;;
	--pool|-p)
	    linkpooldirectories;
	    exit 0;
	    ;;
	--service|-s)
	    addservice;
	    exit 0;
	    ;;
	--noservice|-n)
	    removeservice;
	    exit 0;
	    ;;
        --) shift; break;;
     esac
done


case "$type" in
    sor)
        confname='dpm-httpd-sor.conf'
        ;;
    head-node)
        confname='dpm-httpd-headnode.conf'
        ;;
    disk-node)
        confname='dpm-httpd-diskserver.conf'
        ;;
    *)
	echo "Error [dpm-https-conf]: type <$type> is not known !"
	usage
	exit -1
	;;
esac

FULLSERVERNAME=$(hostname -f)
if [ $(uname -m) = 'x86_64' ]; then
    lib_m='lib64'
else
    lib_m='lib'
fi
sed -e "s/@FULLSERVERNAME@/$FULLSERVERNAME/; s/@lib@/$lib_m/;" $confdir/$confname > $confdir/dpm-httpd.conf
# gridsite module might still be in 'lib' instead of 'lib64'!
if [ "$lib_m" = 'lib64' -a -e '/usr/lib/httpd/modules/mod_gridsite.so' -a ! -e '/usr/lib64/httpd/modules/mod_gridsite.so' ]; then
	sed -i -e 's,/usr/lib64/httpd/modules/mod_gridsite.so,/usr/lib/httpd/modules/mod_gridsite.so,' $confdir/dpm-httpd.conf
fi

testkeys
keypermissions
linkpooldirectories
addservice
exit 0

